<?php
// TODO
// Doing sandbox requires manually entering keys

// PRO - Orders API, digital wallets?


use Square\SquareClient;
use Square\Environment;
use Square\Exceptions\ApiException;

class SPC_Payment_Method_Square extends SPC_Payment_Method {

	private $square; // Square instance to do interact with API
	private $total    = 0;
	private $currency = 'USD';

	public function init() {

		$this->id                    = 'square';
		$this->name                  = __( 'Square', 'sunshine-photo-cart' );
		$this->class                 = get_class( $this );
		$this->description           = __( 'Pay with credit card', 'sunshine-photo-cart' );
		$this->can_be_enabled        = true;
		$this->needs_billing_address = false;

		add_action( 'sunshine_square_connect_display', array( $this, 'square_connect_display' ) );
		add_action( 'admin_init', array( $this, 'square_connect_return' ) );
		add_action( 'admin_init', array( $this, 'square_disconnect' ) );
		add_action( 'admin_init', array( $this, 'square_refresh_locations' ) );
		add_action( 'admin_init', array( $this, 'square_refresh_token' ) );

		if ( ! $this->is_active() || ! $this->is_allowed() ) {
			return;
		}

		add_action( 'sunshine_square_access_token_refresh', array( $this, 'refresh_token' ) );

		add_action( 'wp', array( $this, 'init_setup' ) );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
		//add_filter( 'sunshine_checkout_field_payment_method_square', array( $this, 'buttons' ) );

		add_action( 'wp_ajax_nopriv_sunshine_square_init_order', array( $this, 'init_order' ) );
		add_action( 'wp_ajax_sunshine_square_init_order', array( $this, 'init_order' ) );

		add_action( 'sunshine_checkout_create_order', array( $this, 'create_order' ), 10, 2 );

		// add_action( 'template_redirect', array( $this, 'square_return_listener' ), 999 );
		// add_action( 'template_redirect', array( $this, 'webhooks' ) );

		// add_filter( 'sunshine_order_transaction_url', array( $this, 'transaction_url' ) );

		add_filter( 'sunshine_admin_order_tabs', array( $this, 'admin_order_tab' ), 10, 2 );
		add_action( 'sunshine_admin_order_tab_square', array( $this, 'admin_order_tab_content_square' ) );

		add_action( 'sunshine_order_actions', array( $this, 'order_actions' ), 10, 2 );
		add_action( 'sunshine_order_actions_options', array( $this, 'order_actions_options' ) );
		add_action( 'sunshine_order_process_action_square_refund', array( $this, 'process_refund' ) );

		add_action( 'sunshine_checkout_validation', array( $this, 'checkout_validation' ) );

	}

	/* ADMIN */
	public function options( $options ) {

		foreach ( $options as &$option ) {
			if ( $option['id'] == 'square_header' && $this->get_application_fee_percent() > 0 ) {
				$option['description'] = sprintf( __( 'Note: You are using the free Square payment gateway integration. This includes an additional %s%% fee for payment processing on each order that goes to Sunshine Photo Cart in addition to Square processing fees. This added fee is removed by using the Square Pro add-on.', 'sunshine-photo-cart' ), $this->get_application_fee_percent() ) . ' <a href="https://www.sunshinephotocart.com/addon/square/?utm_source=plugin&utm_medium=link&utm_campaign=square" target="_blank">' . __( 'Learn more', 'sunshine-photo-cart' ) . '</a>';
			}
		}

		// TODO: Better descriptions and help on how to get the keys in square

		$options[] = array(
			'name'    => __( 'Mode', 'sunshine-photo-cart' ),
			'id'      => $this->id . '_mode',
			'type'    => 'radio',
			'options' => array(
				'live' => __( 'Live', 'sunshine-photo-cart' ),
				'test' => __( 'Sandbox', 'sunshine-photo-cart' ),
			),
			'default' => 'live',
		);

		$options[] = array(
			'name'       => __( 'Connect (Production)', 'sunshine-photo-cart' ),
			'id'         => $this->id . '_connect_live',
			'type'       => 'square_connect',
			'conditions' => array(
				array(
					'field'   => $this->id . '_mode',
					'compare' => '==',
					'value'   => 'live',
					'action'  => 'show',
				),
			),
			'hide_system_info' => true,
		);

		$options[] = array(
			'name'        => __( 'Sandbox Settings', 'sunshine-photo-cart' ),
			'id'          => $this->id . '_sandbox_settings',
			'type'        => 'header',
			'description' => sprintf( __( 'Sandbox details can be created at: %s', 'sunshine-photo-cart' ), '<a href="https://developer.squareup.com/apps" target="_blank">https://developer.squareup.com/apps</a>' ),
			'conditions'  => array(
				array(
					'field'   => $this->id . '_mode',
					'compare' => '==',
					'value'   => 'test',
					'action'  => 'show',
				),
			),
			'hide_system_info' => true,
		);

		$options[] = array(
			'name'       => __( 'Sandbox Application ID', 'sunshine-photo-cart' ),
			'id'         => $this->id . '_application_id_test',
			'type'       => 'text',
			'conditions' => array(
				array(
					'field'   => $this->id . '_mode',
					'compare' => '==',
					'value'   => 'test',
					'action'  => 'show',
				),
			),
			'hide_system_info' => true,
		);
		$options[] = array(
			'name'       => __( 'Sandbox Access Token', 'sunshine-photo-cart' ),
			'id'         => $this->id . '_access_token_test',
			'type'       => 'text',
			'conditions' => array(
				array(
					'field'   => $this->id . '_mode',
					'compare' => '==',
					'value'   => 'test',
					'action'  => 'show',
				),
			),
			'hide_system_info' => true,
		);

		if ( $this->get_access_token( 'live' ) ) {
			$locations        = array( '' => __( 'Use default location', 'sunshine-photo-cart' ) );
			$square_locations = $this->get_option( 'locations_live' );
			if ( ! empty( $square_locations ) && is_array( $square_locations ) ) {
				$locations = array_merge( $locations, $square_locations );
			}
			$options[] = array(
				'name'        => __( 'Location (Production)', 'sunshine-photo-cart' ),
				'id'          => $this->id . '_location_id_live',
				'type'        => 'select',
				'options'     => $locations,
				'description' => '<a href="' . wp_nonce_url( admin_url( 'admin.php?sunshine_square_refresh_locations' ), 'sunshine_square_refresh_locations' ) . '">' . __( 'Refresh locations', 'sunshine-photo-cart' ) . '</a>',
				'conditions'  => array(
					array(
						'field'   => $this->id . '_mode',
						'compare' => '==',
						'value'   => 'live',
						'action'  => 'show',
					),
				),
				'hide_system_info' => true,
			);
		}

		if ( $this->get_access_token( 'test' ) ) {
			$locations        = array( '' => __( 'Use default location', 'sunshine-photo-cart' ) );
			$square_locations = $this->get_option( 'locations_test' );
			if ( ! empty( $square_locations ) && is_array( $square_locations ) ) {
				$locations = array_merge( $locations, $square_locations );
			}
			$options[] = array(
				'name'        => __( 'Sandbox Location', 'sunshine-photo-cart' ),
				'id'          => $this->id . '_location_id_test',
				'type'        => 'select',
				'options'     => $locations,
				'description' => '<a href="' . wp_nonce_url( admin_url( 'admin.php?sunshine_square_refresh_locations' ), 'sunshine_square_refresh_locations' ) . '">' . __( 'Refresh locations', 'sunshine-photo-cart' ) . '</a>',
				'conditions'  => array(
					array(
						'field'   => $this->id . '_mode',
						'compare' => '==',
						'value'   => 'test',
						'action'  => 'show',
					),
				),
				'hide_system_info' => true,
			);
		}

		return $options;
	}

	function square_connect_display( $field ) {

		if ( $field['id'] == 'square_connect_live' ) {
			$mode = 'live';
		} else {
			$mode = 'test';
		}

		$access_token = $this->get_access_token( $mode );

		if ( $access_token ) {
			?>

			<p><a href="<?php echo wp_nonce_url( admin_url( 'admin.php?sunshine_square_disconnect' ), 'sunshine_square_disconnect' ); ?>" class="button"><?php _e( 'Disconnect from Square', 'sunshine-photo-cart' ); ?></a></p>
			<p><a href="<?php echo wp_nonce_url( admin_url( 'admin.php?sunshine_square_refresh_token' ), 'sunshine_square_refresh_token' ); ?>"><?php _e( 'Refresh connection', 'sunshine-photo-cart' ); ?></a></p>

	<?php } else { ?>

			<p><a href="https://www.sunshinephotocart.com/?square_connect=1&mode=<?php echo $mode; ?>&return_url=<?php echo admin_url( 'admin.php?sunshine_square_connect_return' ); ?>" class="button"><?php _e( 'Connect to Square', 'sunshine-photo-cart' ); ?></a></p>

			<?php
	}

	}

	function square_connect_return() {

		if ( ! isset( $_GET['sunshine_square_connect_return'] ) ) {
			return false;
		}

		if ( empty( $_GET['access_token'] ) || empty( $_GET['refresh_token'] ) ) {
			SPC()->notices->add_admin( 'square_connect_fail', __( 'Square could not be connected', 'sunshine-photo-cart' ), 'error' );
			wp_redirect( admin_url( 'admin.php?page=sunshine&section=payment_methods&payment_method=square' ) );
			exit;
		}

		if ( isset( $_GET['mode'] ) && $_GET['mode'] == 'live' ) {
			$mode = 'live';
		} else {
			$mode = 'test';
		}

		$access_token  = $this->secure_token( sanitize_text_field( $_GET['access_token'] ), 'e' );
		$refresh_token = $this->secure_token( sanitize_text_field( $_GET['refresh_token'] ), 'e' );
		$this->update_option( 'access_token_' . $mode, $access_token );
		$this->update_option( 'refresh_token_' . $mode, $refresh_token );
		$this->update_option( 'merchant_id_' . $mode, sanitize_text_field( $_GET['merchant_id'] ) );
		$this->update_option( 'token_date_' . $mode, current_time( 'timestamp' ) );
		$this->update_option( 'mode', $mode );
		SPC()->log( 'Square connected' );

		$this->get_locations( $mode );

		// TODO: Establish cron
		if ( ! wp_next_scheduled( 'sunshine_square_access_token_refresh' ) ) {
			$result = wp_schedule_event( current_time( 'timestamp' ), 'weekly', 'sunshine_square_access_token_refresh' );
			if ( is_wp_error( $result ) ) {
				SPC()->log( 'Square access token refresh not scheduled' );
			} else {
				SPC()->log( 'Square access token refresh scheduled' );
			}
		}

		SPC()->notices->add_admin( 'square_connected', __( 'Square has successfully been connected', 'sunshine-photo-cart' ), 'success' );

		wp_redirect( admin_url( 'admin.php?page=sunshine&section=payment_methods&payment_method=square' ) );
		exit;

	}

	function square_disconnect() {

		if ( ! isset( $_GET['sunshine_square_disconnect'] ) || ! check_admin_referer( 'sunshine_square_disconnect' ) ) {
			return;
		}

		$this->setup();

		$mode = $this->get_mode();

		// Disconnect on sunshinephotocart.com
		$request = array(
			'body'    => array(
				'merchant_id' => $this->get_merchant_id(),
				//'access_token' => $this->get_access_token(),
			),
			'timeout' => 45,
		);

		// make the request
		$response = wp_remote_post( 'https://www.sunshinephotocart.com/?square_disconnect=1', $request );
		if ( is_wp_error( $response ) ) {
			SPC()->notices->add_admin( 'square_failed_disconnect', $response->get_error_message(), 'error' );
			return false;
		}

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( isset( $body->error ) || isset( $body->data->error ) ) {
			SPC()->log( sprintf( __( 'Failed to remove association from Sunshine Photo Cart: %s', 'sunshine-photo-cart' ), $body->data->error ) );
			SPC()->notices->add_admin( 'square_failed_disconnect', sprintf( __( 'Failed to remove association from Sunshine Photo Cart: %s', 'sunshine-photo-cart' ), $body->data->error ), 'error' );
		}

		$this->update_option( 'access_token_' . $mode, '' );
		$this->update_option( 'refresh_token_' . $mode, '' );
		$this->update_option( 'merchant_id_' . $mode, '' );
		$this->update_option( 'locations_' . $mode, '' );
		$this->update_option( 'location_id_' . $mode, '' );
		$this->update_option( 'token_date_' . $mode, '' );
		SPC()->log( 'Square disconnected' );

		wp_clear_scheduled_hook( 'sunshine_square_access_token_refresh' );
		SPC()->log( 'Square cron unscheduled' );

		SPC()->notices->add_admin( 'square_disconnected_success', __( 'Square has successfully been disconnected', 'sunshine-photo-cart' ), 'success' );

		wp_redirect( admin_url( 'admin.php?page=sunshine&section=payment_methods&payment_method=square' ) );
		exit;

	}

	function square_refresh_locations() {

		if ( ! isset( $_GET['sunshine_square_refresh_locations'] ) || ! check_admin_referer( 'sunshine_square_refresh_locations' ) ) {
			return;
		}

		SPC()->log( 'Square locations manually refresh triggered' );
		$this->get_locations();

		wp_redirect( admin_url( 'admin.php?page=sunshine&section=payment_methods&payment_method=square' ) );
		exit;

	}

	private function get_locations() {

		$this->setup();

		try {

			$apiResponse = $this->square->getLocationsApi()->listLocations();

			if ( $apiResponse->isSuccess() ) {
				$result    = $apiResponse->getResult();
				$locations = array();
				foreach ( $result->getLocations() as $location ) {
					$locations[ $location->getId() ] = $location->getName() . ' (' . $location->getAddress()->getAddressLine1() . ')';
				}
				$this->update_option( 'locations_' . $this->get_mode(), $locations );
				SPC()->notices->add_admin( 'square_location_refresh', __( 'Locations refreshed from Square', 'sunshine-photo-cart' ) );
				SPC()->log( 'Square locations updated' );
			} else {
				$errors = $apiResponse->getErrors();
				SPC()->notices->add_admin( 'square_location_refresh_fail', __( 'Could not get locations from Square', 'sunshine-photo-cart' ), 'error' );
				SPC()->log( 'Square could not update locations: ' . print_r( $errors, 1 ) );
			}

		} catch ( ApiException $e ) {
			SPC()->notices->add_admin( 'square_location_refresh_fail', $e->getMessage(), 'error' );
			SPC()->log( 'Could not get locations: ' . $e->getMessage() );
		}

	}

	function square_refresh_token() {

		if ( ! isset( $_GET['sunshine_square_refresh_token'] ) || ! check_admin_referer( 'sunshine_square_refresh_token' ) ) {
			return;
		}

		$result = $this->refresh_token();
		wp_redirect( admin_url( 'admin.php?page=sunshine&section=payment_methods&payment_method=square' ) );
		exit;

	}


	public function refresh_token() {

		$request = array(
			'body'    => array(
				'refresh_token' => $this->get_refresh_token(),
			),
			'timeout' => 45,
		);

		// make the request
		$response = wp_remote_post( 'https://www.sunshinephotocart.com/?square_connect_refresh=1', $request );
		if ( is_wp_error( $response ) ) {
			SPC()->notices->add_admin( 'square_failed_refresh_connect', $response->get_error_message(), 'error' );
			return false;
		}

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( isset( $body->error ) || isset( $body->data->error ) ) {
			SPC()->log( sprintf( __( 'Failed to refresh Square token: %s', 'sunshine-photo-cart' ), $body->data->error ) );
			SPC()->notices->add_admin( 'square_failed_refresh', sprintf( __( 'Failed to refresh Square token: %s', 'sunshine-photo-cart' ), $body->data->error ), 'error' );
			return false;
		}

		if ( $body->success ) {
			$mode          = $this->get_mode();
			$access_token  = $this->secure_token( sanitize_text_field( $body->data->access_token ), 'e' );
			$refresh_token = $this->secure_token( sanitize_text_field( $body->data->refresh_token ), 'e' );
			$this->update_option( 'access_token_' . $mode, $access_token );
			$this->update_option( 'refresh_token_' . $mode, $refresh_token );
			$this->update_option( 'merchant_id_' . $mode, $body->data->merchant_id );
			$this->update_option( 'token_date_' . $mode, current_time( 'timestamp' ) );
			SPC()->notices->add_admin( 'square_token_refresh', __( 'Square token successfully refreshed', 'sunshine-photo-cart' ), 'success' );
		}

	}

	private function secure_token( $string, $method = 'e' ) {

		// Set default output value
		$output = null;

		// Set secret keys
		$secret_key = wp_salt();
		$secret_iv  = wp_salt( 'secure_auth' );
		$key        = hash( 'sha256', $secret_key );
		$iv         = substr( hash( 'sha256', $secret_iv ), 0, 16 );

		// Check whether encryption or decryption
		if ( $method == 'e' ) {
			$output = base64_encode( openssl_encrypt( $string, 'AES-256-CBC', $key, 0, $iv ) );
		} elseif ( $method == 'd' ) {
			$output = openssl_decrypt( base64_decode( $string ), 'AES-256-CBC', $key, 0, $iv );
		}

		return $output;

	}

	/* PUBLIC */
	public function init_setup() {

		if ( ! is_sunshine_page( 'checkout' ) ) {
			return;
		}

		$this->setup();

	}

	private function setup( $mode = '' ) {

		if ( empty( $mode ) ) {
			$mode = $this->get_mode();
		}

		if ( empty( $this->get_access_token( $mode ) ) ) {
			return false;
		}

		$this->square = new SquareClient(
			array(
				'accessToken' => $this->get_access_token( $mode ),
				'environment' => ( $this->get_mode() == 'live' ) ? Environment::PRODUCTION : Environment::SANDBOX,
				'squareVersion' => '2023-10-18',
			)
		);

		if ( empty( SPC()->cart ) ) {
			SPC()->cart->setup();
		}
		$this->total    = 100 * SPC()->cart->get_total();
		$this->currency = SPC()->get_option( 'currency' );

	}

	private function get_mode() {
		return $this->get_option( 'mode' );
	}

	private function get_merchant_id( $mode = '' ) {
		if ( empty( $mode ) ) {
			$mode = $this->get_mode();
		}
		$merchant_id = $this->get_option( 'merchant_id_' . $mode );
		return $merchant_id;
	}

	private function get_access_token( $mode = '' ) {
		if ( empty( $mode ) ) {
			$mode = $this->get_mode();
		}
		$access_token = $this->get_option( 'access_token_' . $mode );
		if ( $mode == 'live' ) {
			$access_token = $this->secure_token( $access_token, 'd' );
		}
		return $access_token;
	}

	private function get_refresh_token( $mode = '' ) {
		if ( empty( $mode ) ) {
			$mode = $this->get_mode();
		}
		$access_token = $this->get_option( 'refresh_token_' . $mode );
		$access_token = $this->secure_token( $access_token, 'd' );
		return $access_token;
	}

	private function get_location_id( $mode = '' ) {
		if ( empty( $mode ) ) {
			$mode = $this->get_mode();
		}
		$location_id = $this->get_option( 'location_id_' . $mode );
		if ( empty( $location_id ) ) {
			$locations = $this->get_option( 'locations_' . $mode );
			if ( ! empty( $locations ) && is_array( $locations ) ) {
				$location_id = array_key_first( $locations );
			}
		}
		return $location_id;
	}

	public function get_application_id() {
		if ( $this->get_mode() == 'live' ) {
			//return 'sandbox-sq0idb-aNDJReIdhctf2o3OZA0FTA';
			return 'sq0idp-19QAyk7l68V7ymxc2Fl9EQ';
		}
		return $this->get_option( 'application_id_test' );
	}

	public function is_allowed() {
		if ( ! empty( $this->get_access_token() ) && ! empty( $this->get_application_id() ) ) {
			return true;
		}
		return false;
	}

	public function enqueue_scripts() {

		if ( ! is_sunshine_page( 'checkout' ) ) {
			return false;
		}

		if ( $this->get_mode() == 'live' ) {
			wp_enqueue_script( 'sunshine-square', 'https://web.squarecdn.com/v1/square.js' );
		} else {
			wp_enqueue_script( 'sunshine-square', 'https://sandbox.web.squarecdn.com/v1/square.js' );
		}
		wp_enqueue_script( 'sunshine-square-processing', SUNSHINE_PHOTO_CART_URL . '/assets/js/square-processing.js', array( 'jquery' ), '', true );
		wp_localize_script(
			'sunshine-square-processing',
			'spc_square_vars',
			array(
				'application_id' => $this->get_application_id(),
				'location_id'    => $this->get_location_id(),
				'ajax_url'       => admin_url( 'admin-ajax.php' ),
				'security'       => wp_create_nonce( 'sunshine_square' ),
			)
		);

	}

	public function get_fields() {

		ob_start();

		if ( $this->get_mode() == 'test' ) {
			echo '<div class="sunshine--payment--test">' . __( 'This will be processed as a test payment and no real money will be exchanged', 'sunshine-photo-cart' ) . '</div>';
		}
		?>

		<div id="sunshine-square-payment">
			<div id="sunshine-square-payment-fields"></div>
			<div id="sunshine-square-payment-errors"></div>
		</div>

		<?php
		$output = ob_get_contents();
		ob_end_clean();
		return $output;

	}

	private function get_application_fee_percent() {
		return floatval( apply_filters( 'sunshine_square_application_fee_percent', 5 ) );
	}

	private function get_application_fee_amount() {

		$percentage = $this->get_application_fee_percent();

		// Some countries do not allow us to use application fees. If we are in one of those
		// countries, we should set the percentage to 0. This is a temporary fix until we
		// have a better solution or until all countries allow us to use application fees.
		$country                             = SPC()->get_option( 'country' );
		$countries_to_allow_application_fees = array(
			'US',
		);
		if ( ! in_array( $country, $countries_to_allow_application_fees ) ) {
			$percentage = 0;
		}

		if ( $percentage <= 0 ) {
			return 0;
		}

		$percentage = floatval( $percentage );

		return round( $this->total * ( $percentage / 100 ), 2 );

	}

	public function get_idempotency_key( $key_input = '', $append_key_input = true ) {

		if ( '' === $key_input ) {
			$key_input = uniqid( '', false );
		}

		return substr( apply_filters( 'sunshine_square_idempotency_key', sha1( get_option( 'siteurl' ) . $key_input ) . ( $append_key_input ? ':' . $key_input : '' ) ), -40 );
	}

	public function init_order() {

		if ( empty( $_POST['security'] ) ) {
			wp_send_json_error( array( 'reasons' => __( 'Failed to pass security', 'sunshine-photo-cart' ) ) );
		}

		if ( empty( $_POST['source_id'] ) ) {
			wp_send_json_error( array( 'reasons' => __( 'No source ID', 'sunshine-photo-cart' ) ) );
		}

		$source_id = sanitize_text_field( $_POST['source_id'] );

		$this->setup();

		$body = new \Square\Models\CreatePaymentRequest(
			$source_id,
			$this->get_idempotency_key( $source_id ),
		);

		$amount_money = new \Square\Models\Money();
		$amount_money->setAmount( $this->total );
		$amount_money->setCurrency( $this->currency );
		$body->setAmountMoney( $amount_money );

		$app_fee = $this->get_application_fee_amount();
		if ( $app_fee ) {
			$app_fee_money = new \Square\Models\Money();
			$app_fee_money->setAmount( $app_fee );
			$app_fee_money->setCurrency( 'USD' );
			$body->setAppFeeMoney( $app_fee_money );
		}

		$body->setAutocomplete( true );
		// $body->setCustomerId( 'W92WH6P11H4Z77CTET0RNTGFW8' );
		$body->setLocationId( $this->get_location_id() );
		// $body->setReferenceId( '123456' );
		// $body->setNote( 'Brief description' );

		$api_response = $this->square->getPaymentsApi()->createPayment( $body );

		$reasons = array();

		if ( $api_response->isSuccess() ) {
			$result  = $api_response->getResult();
			$payment = $result->getPayment();
			wp_send_json_success( array( 'payment_id' => $payment->getId() ) );
		} else {
			$errors = $api_response->getErrors();
			foreach ( $errors as $error ) {
				$reasons[] = sprintf(
					'%s: %s, %s',
					$error->getCategory(),
					$error->getCode(),
					$error->getDetail()
				);
			}
			wp_send_json_error( array( 'reasons' => $reasons ) );
		}

	}

	public function create_order( $order, $data ) {

		if ( empty( $data['payment_method'] ) || $data['payment_method'] != $this->id || empty( $_POST['square_payment_id'] ) ) {
			return;
		}

		$this->setup();

		$payment_id = sanitize_text_field( $_POST['square_payment_id'] );

		$api_response = $this->square->getPaymentsApi()->getPayment( $payment_id );
		if ( $api_response->isSuccess() ) {
			$payment_data = $api_response->getResult()->jsonSerialize();
			$payment      = $payment_data['payment'];
			if ( $payment->getAppFeeMoney() ) {
				$order->add_meta_value( 'square_app_fee', $payment->getAppFeeMoney()->getAmount() / 100 );
			}
		}

		$order->add_meta_value( 'square_payment_id', $payment_id );
		$order->set_status( 'new' );

		// $this->create_square_order( $order );
	}

	public function get_transaction_id( $order ) {
		return $order->get_meta_value( 'square_payment_id' );
	}

	public function get_transaction_url( $order ) {
		if ( $order->get_payment_method() == 'square' ) {
			$transaction_id = $this->get_transaction_id( $order );
			if ( $transaction_id ) {
				$mode             = $order->get_mode();
				$transaction_url  = ( $mode == 'test' ) ? 'https://squareupsandbox.com/dashboard/sales/transactions/' : 'https://squareup.com/dashboard/sales/transactions/';
				$transaction_url .= $transaction_id;
				return $transaction_url;
			}
		}
		return false;
	}

	public function get_app_fee( $order ) {
		return $order->get_meta_value( 'square_app_fee' );
	}

	public function admin_order_tab( $tabs, $order ) {
		if ( $order->get_payment_method() == $this->id ) {
			$tabs['square'] = __( 'Square', 'sunshine-photo-cart' );
		}
		return $tabs;
	}

	public function admin_order_tab_content_square( $order ) {

		echo '<table class="sunshine-data">';
		echo '<tr><th>' . __( 'Transaction ID', 'sunshine-photo-cart' ) . '</th>';
		echo '<td>' . $this->get_transaction_id( $order ) . '</td></tr>';

		$application_fee_amount = $this->get_app_fee( $order );
		if ( $application_fee_amount ) {
			echo '<tr>';
			echo '<th>' . __( 'Application Fee Amount (To Sunshine)', 'sunshine-photo-cart' ) . '</th>';
			echo '<td>' . sunshine_price( $application_fee_amount ) . ' (<a href="https://www.sunshinephotocart.com/upgrade/?utm_source=plugin&utm_medium=link&utm_campaign=stripe" target="_blank">' . __( 'Upgrade to remove this fee on future transactions', 'sunshine-photo-cart' ) . '</a>)' . '</td>';
			echo '</tr>';
		}

		echo '</table>';

	}


	function order_actions( $actions, $post_id ) {
		$order = new SPC_Order( $post_id );
		if ( $order->get_payment_method() == $this->id ) {
			$actions['square_refund'] = __( 'Refund payment in Square', 'sunshine-photo-cart' );
		}
		return $actions;
	}

	function order_actions_options( $order ) {
		?>
		<div id="square-refund-order-actions" style="display: none;">
			<p><label><input type="checkbox" name="square_refund_notify" value="yes" checked="checked" /> <?php _e( 'Notify customer via email', 'sunshine-photo-cart' ); ?></label></p>
			<p><label><input type="checkbox" name="square_refund_full" value="yes" checked="checked" /> <?php _e( 'Full refund', 'sunshine-photo-cart' ); ?></label></p>
			<p id="square-refund-amount" style="display: none;"><label><input type="number" name="square_refund_amount" step=".01" size="6" style="width:100px" max="<?php echo esc_attr( $order->get_total_minus_refunds() ); ?>" value="<?php echo esc_attr( $order->get_total_minus_refunds() ); ?>" /> <?php _e( 'Amount to refund', 'sunshine-photo-cart' ); ?></label></p>
		</div>
		<script>
			jQuery( 'select[name="sunshine_order_action"]' ).on( 'change', function(){
				let selected_action = jQuery( 'option:selected', this ).val();
				if ( selected_action == 'square_refund' ) {
					jQuery( '#square-refund-order-actions' ).show();
				} else {
					jQuery( '#square-refund-order-actions' ).hide();
				}
			});
			jQuery( 'input[name="square_refund_full"]' ).on( 'change', function(){
				if ( !jQuery(this).prop( "checked" ) ) {
					jQuery( '#square-refund-amount' ).show();
				} else {
					jQuery( '#square-refund-amount' ).hide();
				}
			});
		</script>
		<?php
	}

	// TODO: Process refund
	function process_refund( $order_id ) {

		$order = new SPC_Order( $order_id );

		$refund_amount = $order->get_total_minus_refunds();

		if ( ! empty( $_POST['square_refund_amount'] ) && $_POST['square_refund_amount'] < $refund_amount ) {
			$refund_amount = sanitize_text_field( $_POST['square_refund_amount'] );
		}

		$refund_amount_square = $refund_amount * 100; // Lose decimals because Square

		$this->setup( $order->get_mode() );

		$payment_id = $this->get_transaction_id( $order );

		$amount_money = new \Square\Models\Money();
		$amount_money->setAmount( $refund_amount_square );
		$amount_money->setCurrency( $order->get_currency() );

		$app_fee = $this->get_app_fee( $order );
		if ( $app_fee ) {
			$refund_percent = $refund_amount / $order->get_total_minus_refunds();
			$refund_app_fee = ( $app_fee * $refund_percent ) * 100;
			$app_fee_money  = new \Square\Models\Money();
			$app_fee_money->setAmount( $refund_app_fee );
			$app_fee_money->setCurrency( $order->get_currency() );
		}

		$idempotency_key = md5( time() . $payment_id );

		$body = new \Square\Models\RefundPaymentRequest( $idempotency_key, $amount_money );
		$body->setAppFeeMoney( $app_fee_money );
		$body->setPaymentId( $payment_id );

		$api_response = $this->square->getRefundsApi()->refundPayment( $body );
		if ( $api_response->isSuccess() ) {
			SPC()->notices->add_admin( 'square_refund_success_' . $payment_id, sprintf( __( 'Refund has been processed for %s', 'sunshine-photo-cart' ), sunshine_price( $refund_amount ) ) );
			$order->set_status( 'refunded' );
			$order->add_refund( $refund_amount );
			$order->update();
			if ( ! empty( $refund_app_fee ) ) {
				$order->add_log( sprintf( __( 'Sunshine contributed %s towards refund from processing fees', 'sunshine-photo-cart' ), sunshine_price( $refund_app_fee / 100 ) ) );
			}
		} else {
			$errors = $api_response->getErrors();
			foreach ( $errors as $error ) {
				$reasons[] = sprintf(
					'%s: %s, %s',
					$error->getCategory(),
					$error->getCode(),
					$error->getDetail()
				);
			}
			SPC()->notices->add_admin( 'square_refund_fail_' . $payment_id, sprintf( __( 'Could not refund payment in Square: %s', 'sunshine-photo-cart' ), print_r( $reasons, 1 ) ), 'error' );
			$order->add_log( sprintf( __( 'Could not refund payment in Square: %s', 'sunshine-photo-cart' ), print_r( $reasons, 1 ) ) );
		}

	}

	public function mode( $mode, $order ) {
		if ( $order->get_payment_method() == 'square' ) {
			return ( $this->get_mode() == 'live' ) ? 'live' : 'test';
		}
		return $mode;
	}

	public function checkout_validation( $section ) {
		if ( $section == 'payment' && SPC()->cart->get_total() > 0 && SPC()->cart->get_checkout_data_item( 'payment_method' ) == 'square' ) {
			if ( empty( $_POST['square_payment_id'] ) ) {
				SPC()->cart->add_error( __( 'Invalid payment', 'sunshine-photo-cart' ) );
			}
		}
	}

}
