<?php

class SPC_Shipping_Method_Flat_Rate extends SPC_Shipping_Method {

	public function init() {
		$this->id            = 'flat_rate';
		$this->name          = __( 'Flat Rate', 'sunshine-photo-cart' );
		$this->class         = 'SPC_Shipping_Method_Flat_Rate';
		$this->description   = '';
		$this->can_be_cloned = true;
	}

	public function get_price() {

		$price_type = SPC()->get_option( $this->id . '_price_type_' . $this->instance_id );
		if ( empty( $price_type ) ) {
			return parent::get_price();
		}

		$product_shipping = 0;
		$cart_items = SPC()->cart->get_cart_items();
		if ( ! empty( $cart_items ) ) {
			foreach ( SPC()->cart->get_cart_items() as $item ) {
				// Add product shipping fee if exists.
				$item_shipping = $item->product->get_shipping();
				if ( $item_shipping ) {
					$product_shipping += $item_shipping * $item->get_qty();
				}
			}
		}

		$price = SPC()->get_option( $this->id . '_price_' . $this->instance_id );
		if ( $price_type == 'cart' ) {
			return floatval( $price ) + floatval( $product_shipping );
		}

		if ( ! empty( $this->instance_id ) ) {

			if ( ! empty( $cart_items ) ) {

				$total = 0;
				foreach ( SPC()->cart->get_cart_items() as $item ) {

					// Determine how we calculate shipping fee: line item or qty
					if ( $price_type == 'line' ) {
						$total += $price;
					} elseif ( $price_type == 'qty' ) {
						$total += $price * $item->get_qty();
					}

				}

				return $total + $product_shipping;

			}

			return $price + $product_shipping;

		}

		return false;

	}

	public function is_allowed() {

		if ( empty( $this->instance_id ) ) {
			return false;
		}

		$allowed = apply_filters( 'sunshine_shipping_flat_rate_allowed', true, $this );

		return $allowed;

	}


}

$sunshine_shipping_flat_rate = new SPC_Shipping_Method_Flat_Rate();
